% Excercises with Loops and Vectorizing

%% For Loop
% using a for loop, create a vector that has the squares of the numbers 1 to 10

%% If Loop
% Make a sine wave, and flatten the tops
f=10; % define frequency
t=0.001:0.001:1; % make time base
sine_wave = sin(2*pi*f*t);
plot(sine_wave);
% Now use an if loop to find all values >.6 and set them to .6
    % Note -- looping is an inefficient way to do this...  
    %  Could you use indexing?
% Use HOLD and plot clipped wave over the original wave

%% Sample if and for loops
% Preallocate a matrix
nrows = 10;
ncols = 10;
myData = ones(nrows, ncols);

% Loop through the matrix
for r = 1:nrows
   for c = 1:ncols

      if r == c
         myData(r,c) = 2;
      elseif abs(r - c) == 1
         myData(r,c) = -1;
      else
         myData(r,c) = 0;
      end

   end
end

% Examine myData before and after the loop to see what happens

%% Switch Demo
mynumber = input('Enter a number:');

switch mynumber
    case -1
        disp('negative one');
    case 0
        disp('zero');
    case 1
        disp('positive one');
    otherwise
        disp('other value');
end

% Add another case to evaluate if something is really big (e.g., > 1000000)
% Give it a useful message
% try it out

%% While loop
% find first integer n for which factorial n is greater than 100000
n = 1;
nFactorial = 1;
while nFactorial < 100000
    n = n + 1;
    nFactorial = nFactorial * n;  % could also use built-in Matlab function: factorial(n)
end
disp([num2str(n) ' factorial is greater than 100000']);

% make a while loop that displays integers in increasing order until 10
% Is this efficient?

%% For vs Vectorizing  a demo -- all you need to do is run the code and inspect the results
% get set up
clear all
load sampleEEGdata.mat
mockdata = EEG.data; % channels by timepoints by epochs
n_chans = size(mockdata,1);
n_times = size(mockdata,2);
n_epochs = size(mockdata,3);
% Initialized mean matrices to hold results  (BONUS:  rerun without to compare)
meandata = nan(n_chans,n_times);
meandata_vec = nan(n_chans,n_times);
meandata_singlstep = nan(n_chans,n_times);


for n_chan = 1:size(mockdata,1)
    % for loop on timepoints
    tic
    for n_time = 1:size(mockdata,2)  % loop on timepoints and take across epochs, on timepoint at a time
        meandata(n_chan,n_time) = mean(mockdata(n_chan,n_time),3);
    end
    For_time(n_chan) = toc;
    
    % Multiply using vectorizing
    tic
    meandata_vec(n_chan,:) = mean(mockdata(n_chan,:,:),3); % for one chan, take mean across epochs (no loop on time points)
    Vectorize_time(n_chan) = toc;
end;

% now super vectorize it
tic
meandata_singlestep = mean(mockdata,3);  % Just take the mean across the relevant dimension (epochs)
SingleStep_time = toc/size(mockdata,1);  % divide by n_channels to get per-channel time to compare to other time estimates above

% Make a scatter plot
scatter(For_time, Vectorize_time);
% Make x and y axis the same, and scale to larger range (the for loop times are bigger)
ylim([0 max(For_time)]);
xlim([0 max(For_time)]);
hold on  % Now we don't erase data, but plot on top
scatter(SingleStep_time,SingleStep_time,'or')  % plot a red circle 'or'


